@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of urlwatch (https://thp.io/2008/urlwatch/).
-# Copyright (c) 2008-2019 Thomas Perl <m@thp.io>
+# Copyright (c) 2008-2020 Thomas Perl <m@thp.io>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -39,6 +39,7 @@ import json
 
 from enum import Enum
 from lxml import etree
+from lxml.cssselect import CSSSelector
 
 from .util import TrackSubClasses
 
@@ -154,6 +155,29 @@ class LegacyHooksPyFilter(FilterBase):
             return data
 
 
+class BeautifyFilter(FilterBase):
+    """Beautify HTML"""
+
+    __kind__ = 'beautify'
+
+    def filter(self, data, subfilter=None):
+        import jsbeautifier
+        import cssbeautifier
+        from bs4 import BeautifulSoup as bs
+        soup = bs(data, features="lxml")
+        scripts = soup.find_all('script')
+        for script in scripts:
+            if script.string is not None:
+                beautified_js = jsbeautifier.beautify(script.string)
+                script.string = beautified_js
+        styles = soup.find_all('style')
+        for style in styles:
+            if style.string is not None:
+                beautified_css = cssbeautifier.beautify(style.string)
+                style.string = beautified_css
+        return soup.prettify()
+
+
 class Html2TextFilter(FilterBase):
     """Convert HTML to plaintext"""
 
@@ -195,7 +219,7 @@ class JsonFormatFilter(FilterBase):
         if subfilter is not None:
             indentation = int(subfilter)
         parsed_json = json.loads(data)
-        return json.dumps(parsed_json, sort_keys=True, indent=indentation, separators=(',', ': '))
+        return json.dumps(parsed_json, ensure_ascii=False, sort_keys=True, indent=indentation, separators=(',', ': '))
 
 
 class GrepFilter(FilterBase):
@@ -372,29 +396,28 @@ class LxmlParser:
 
     def __init__(self, filter_kind, subfilter, expr_key):
         self.filter_kind = filter_kind
-        self.expression, self.method = self.parse_subfilter(
-            filter_kind, subfilter, expr_key, self.EXPR_NAMES[filter_kind])
-        self.parser = (etree.HTMLParser if self.method == 'html' else etree.XMLParser)()
-        self.data = ''
-
-    @staticmethod
-    def parse_subfilter(filter_kind, subfilter, expr_key, expr_name):
         if subfilter is None:
-            raise ValueError('Need %s for filtering' % (expr_name,))
+            raise ValueError('Need %s for filtering' % (self.EXPR_NAMES[filter_kind],))
         if isinstance(subfilter, str):
-            expression = subfilter
-            method = 'html'
+            self.expression = subfilter
+            self.method = 'html'
+            self.exclude = None
+            self.namespaces = None
         elif isinstance(subfilter, dict):
             if expr_key not in subfilter:
-                raise ValueError('Need %s for filtering' % (expr_name,))
-            expression = subfilter[expr_key]
-            method = subfilter.get('method', 'html')
-            if method not in ('html', 'xml'):
-                raise ValueError('%s method must be "html" or "xml", got %r' % (filter_kind, method))
+                raise ValueError('Need %s for filtering' % (self.EXPR_NAMES[filter_kind],))
+            self.expression = subfilter[expr_key]
+            self.method = subfilter.get('method', 'html')
+            self.exclude = subfilter.get('exclude')
+            self.namespaces = subfilter.get('namespaces')
+            if self.method not in ('html', 'xml'):
+                raise ValueError('%s method must be "html" or "xml", got %r' % (filter_kind, self.method))
+            if self.method == 'html' and self.namespaces is not None:
+                raise ValueError('Namespace prefixes only supported with "xml" method.')
         else:
             raise ValueError('%s subfilter must be a string or dict' % (filter_kind,))
-
-        return expression, method
+        self.parser = (etree.HTMLParser if self.method == 'html' else etree.XMLParser)()
+        self.data = ''
 
     def feed(self, data):
         self.data += data
@@ -406,6 +429,60 @@ class LxmlParser:
 
         return etree.tostring(element, pretty_print=True, method=self.method, encoding='unicode', with_tail=False)
 
+    @staticmethod
+    def _remove_element(element):
+        parent = element.getparent()
+        if parent is None:
+            # Do not exclude root element
+            return
+        if isinstance(element, etree._ElementUnicodeResult):
+            if element.is_tail:
+                parent.tail = None
+            elif element.is_text:
+                parent.text = None
+            elif element.is_attribute:
+                del parent.attrib[element.attrname]
+        else:
+            previous = element.getprevious()
+            if element.tail is not None:
+                if previous is not None:
+                    previous.tail = previous.tail + element.tail if previous.tail else element.tail
+                else:
+                    parent.text = parent.text + element.tail if parent.text else element.tail
+            parent.remove(element)
+
+    def _reevaluate(self, element):
+        if self._orphaned(element):
+            return None
+        if isinstance(element, etree._ElementUnicodeResult):
+            parent = element.getparent()
+            if parent is None:
+                return element
+            if element.is_tail:
+                return parent.tail
+            elif element.is_text:
+                return parent.text
+            elif element.is_attribute:
+                return parent.attrib.get(element.attrname)
+        else:
+            return element
+
+    def _orphaned(self, element):
+        if isinstance(element, etree._ElementUnicodeResult):
+            parent = element.getparent()
+            if ((element.is_tail and parent.tail is None)
+                    or (element.is_text and parent.text is None)
+                    or (element.is_attribute and parent.attrib.get(element.attrname) is None)):
+                return True
+            else:
+                element = parent
+        try:
+            tree = element.getroottree()
+            path = tree.getpath(element)
+            return element is not tree.xpath(path, namespaces=self.namespaces)[0]
+        except (ValueError, IndexError):
+            return True
+
     def _get_filtered_elements(self):
         try:
             root = etree.fromstring(self.data, self.parser)
@@ -418,10 +495,19 @@ class LxmlParser:
             root = etree.fromstring(self.data, self.parser)
         if root is None:
             return []
+        excluded_elems = None
         if self.filter_kind == 'css':
-            return root.cssselect(self.expression)
+            selected_elems = CSSSelector(self.expression,
+                                         namespaces=self.namespaces).evaluate(root)
+            excluded_elems = CSSSelector(self.exclude,
+                                         namespaces=self.namespaces).evaluate(root) if self.exclude else None
         elif self.filter_kind == 'xpath':
-            return root.xpath(self.expression)
+            selected_elems = root.xpath(self.expression, namespaces=self.namespaces)
+            excluded_elems = root.xpath(self.exclude, namespaces=self.namespaces) if self.exclude else None
+        if excluded_elems is not None:
+            for el in excluded_elems:
+                self._remove_element(el)
+        return [el for el in map(self._reevaluate, selected_elems) if el is not None]
 
     def get_filtered_data(self):
         return '\n'.join(self._to_string(element) for element in self._get_filtered_elements())
@@ -447,3 +533,32 @@ class XPathFilter(FilterBase):
         lxml_parser = LxmlParser('xpath', subfilter, 'path')
         lxml_parser.feed(data)
         return lxml_parser.get_filtered_data()
+
+
+class RegexSub(FilterBase):
+    """Replace text with regular expressions using Python's re.sub"""
+
+    __kind__ = 're.sub'
+
+    def filter(self, data, subfilter=None):
+        if subfilter is None:
+            raise ValueError('{} needs a subfilter'.format(self.__kind__))
+
+        # Allow for just specifying a regular expression (that will be removed)
+        if isinstance(subfilter, str):
+            subfilter = {'pattern': subfilter}
+
+        # Default: Replace with empty string if no "repl" value is set
+        return re.sub(subfilter.get('pattern'), subfilter.get('repl', ''), data)
+
+
+class SortFilter(FilterBase):
+    """Sort the results before comparison"""
+
+    __kind__ = 'sort'
+
+    def filter(self, data, subfilter=None):
+        data_list = data.splitlines()
+        data_list = sorted(data_list, key=str.casefold)
+
+        return '\n'.join(data_list)
