@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of urlwatch (https://thp.io/2008/urlwatch/).
-# Copyright (c) 2008-2019 Thomas Perl <m@thp.io>
+# Copyright (c) 2008-2020 Thomas Perl <m@thp.io>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -59,6 +59,12 @@ DEFAULT_CONFIG = {
             'minimal': False,
         },
 
+        'markdown': {
+            'details': True,
+            'footer': True,
+            'minimal': False,
+        },
+
         'html': {
             'diff': 'unified',  # "unified" or "table"
         },
@@ -81,7 +87,7 @@ DEFAULT_CONFIG = {
                 'user': '',
                 'port': 25,
                 'starttls': True,
-                'keyring': True,
+                'auth': True,
             },
             'sendmail': {
                 'path': 'sendmail',
@@ -90,7 +96,7 @@ DEFAULT_CONFIG = {
         'pushover': {
             'enabled': False,
             'app': '',
-            'device': '',
+            'device': None,
             'sound': 'spacealarm',
             'user': '',
         },
@@ -107,6 +113,12 @@ DEFAULT_CONFIG = {
             'enabled': False,
             'webhook_url': '',
         },
+        'matrix': {
+            'enabled': False,
+            'homeserver': '',
+            'access_token': '',
+            'room_id': '',
+        },
         'mailgun': {
             'enabled': False,
             'region': 'us',
