@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of urlwatch (https://thp.io/2008/urlwatch/).
-# Copyright (c) 2008-2018 Thomas Perl <m@thp.io>
+# Copyright (c) 2008-2020 Thomas Perl <m@thp.io>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -30,6 +30,7 @@
 
 import concurrent.futures
 import logging
+import difflib
 
 import requests
 
@@ -52,7 +53,8 @@ def run_parallel(func, items):
 
 def run_jobs(urlwatcher):
     cache_storage = urlwatcher.cache_storage
-    jobs = urlwatcher.jobs
+    jobs = [job.with_defaults(urlwatcher.config_storage.config)
+            for job in urlwatcher.jobs]
     report = urlwatcher.report
 
     logger.debug('Processing %d jobs', len(jobs))
@@ -67,9 +69,14 @@ def run_jobs(urlwatcher):
         logger.debug('Using max_tries of %i for %s', max_tries, job_state.job)
 
         if job_state.exception is not None:
-            if isinstance(job_state.exception, NotModifiedError):
+            if job_state.error_ignored:
+                logger.info('Error while executing job %s ignored due to job config', job_state.job)
+            elif isinstance(job_state.exception, NotModifiedError):
                 logger.info('Job %s has not changed (HTTP 304)', job_state.job)
                 report.unchanged(job_state)
+                if job_state.tries > 0:
+                    job_state.tries = 0
+                    job_state.save()
             elif job_state.tries < max_tries:
                 logger.debug('This was try %i of %i for job %s', job_state.tries,
                              max_tries, job_state.job)
@@ -77,19 +84,26 @@ def run_jobs(urlwatcher):
             elif job_state.tries >= max_tries:
                 logger.debug('We are now at %i tries ', job_state.tries)
                 job_state.save()
-                if isinstance(job_state.exception, requests.exceptions.RequestException):
-                    # Instead of a full traceback, just show the HTTP error
-                    job_state.traceback = str(job_state.exception)
-                    report.error(job_state)
-                else:
-                    report.error(job_state)
+                report.error(job_state)
 
         elif job_state.old_data is not None:
-            if job_state.old_data.splitlines() != job_state.new_data.splitlines():
-                report.changed(job_state)
-                job_state.save()
-            else:
+            matched_history_time = job_state.history_data.get(job_state.new_data)
+            if matched_history_time:
+                job_state.timestamp = matched_history_time
+            if matched_history_time or job_state.new_data == job_state.old_data:
                 report.unchanged(job_state)
+                if job_state.tries > 0:
+                    job_state.tries = 0
+                    job_state.save()
+            else:
+                close_matches = difflib.get_close_matches(job_state.new_data, job_state.history_data, n=1)
+                if close_matches:
+                    job_state.old_data = close_matches[0]
+                    job_state.timestamp = job_state.history_data[close_matches[0]]
+                report.changed(job_state)
+                job_state.tries = 0
+                job_state.save()
         else:
             report.new(job_state)
+            job_state.tries = 0
             job_state.save()
